function EDBplotmodel(eddatafile,plotoptions);
% EDBplotmodel - Plots a model which is given in an eddatafile.
%
% Input parameters:
%   eddatafile (optional)   The input file. If not specified, a file
%                           opening window will be presented.
%   plotoptions (optional)  An integer that can give extra options:
%                           if bit0 = 1 (= 1) => plot sources
%                           if bit1 = 1 (= 2) => plot receivers
%                           if bit2 = 1 (= 4) => plot plane normal vectors
%                           if bit3 = 1 (= 8) => print plane numbers
%                           if bit4 = 1 (=16) => print edge numbers
%                           if bit5 = 1 (=32) => print corner numbers
%                           if bit6 = 1 (=64) => print plane numbers using the
%                                                CAD file numbering
%                           if bit7 = 1 (=128)=> print corner numbers using the
%                                                CAD file numbering
%                           Example: the integer 11 = 1011 binary,
%                           so bits 0,1, and 3 are set.
%
% Sources and receivers are taken from an sdatafile and an rdatafile, the file name of
% which is assumed to be similar to the eddatafile.
%
% Uses functions EDBstrpend, EDBmyvalueinput
%
% ----------------------------------------------------------------------------------------------
%   This file is part of the Edge Diffraction Toolbox by Peter Svensson.                       
%                                                                                              
%   The Edge Diffraction Toolbox is free software: you can redistribute it and/or modify       
%   it under the terms of the GNU General Public License as published by the Free Software     
%   Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                              
%   The Edge Diffraction Toolbox is distributed in the hope that it will be useful,       
%   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                              
%   You should have received a copy of the GNU General Public License along with the           
%   Edge Diffraction Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
% ----------------------------------------------------------------------------------------------
% Peter Svensson (svensson@iet.ntnu.no) 8Jul09
%
% EDBplotmodel(eddatafile,plotoptions);

%--------------------------------------------------------------
% Read the eddatafile

if nargin == 0 | (nargin == 1 & isstr(eddatafile) ~= 1),
	[eddatafile,eddatafilepath] = uigetfile('*eddata.mat','Please select the eddatafile')
    [eddatafilepath,temp1,temp2,temp3] = fileparts(eddatafilepath);
	if ~isstr(eddatafile) | isempty(eddatafile),
		return
	end
else,
% 	[eddatafilepath,eddatafile] = EDBstrppath(eddatafile);
	[eddatafilepath,eddatafile,fileext,temp] = fileparts(eddatafile);
    eddatafile = [eddatafile,fileext];
end
Filestem = EDBstrpend(eddatafile,'_eddata');

if nargin == 1
    if isstr(eddatafile) ~= 1,
        plotoptions = eddatafile;
    else,
        plotoptions = 0;    
    end
elseif nargin == 0,
    plotoptions = 0;    
end
plotsources = bitget(plotoptions,1);
plotreceivers = bitget(plotoptions,2);
plotnvecs = bitget(plotoptions,3);
plotplnumbers = bitget(plotoptions,4);
plotednumbers = bitget(plotoptions,5);
plotconumbers = bitget(plotoptions,6);
plotplCADnumbers = bitget(plotoptions,7);
plotcoCADnumbers = bitget(plotoptions,8);

if plotplCADnumbers, plotplnumbers = 1; end
if plotcoCADnumbers, plotconumbers = 1; end

%--------------------------------------------------------------------------
% Load the needed input files

if ~isempty(eddatafilepath)
    eval(['load ',eddatafilepath,filesep,eddatafile])
else
    eval(['load ',eddatafile])    
end
    
ncornersperplanevec = double(ncornersperplanevec);
if plotsources,
    sdatafile = [eddatafilepath,Filestem,'_sdata.mat'];
    if exist(sdatafile) == 2,
        eval(['load ',sdatafile])
    else,
        error(['ERROR: The sdata file named ',sdatafile,' could not be opened'])    
    end
end
if plotreceivers,
    rdatafile = [eddatafilepath,Filestem,'_rdata.mat'];
    if exist(rdatafile) == 2,
        eval(['load ',rdatafile])
    else,
        error(['ERROR: The rdata file named ',rdatafile,' could not be opened'])    
    end
end

if plotplCADnumbers | plotcoCADnumbers,
    cadgeofile = [eddatafilepath,Filestem,'_cadgeo.mat'];
    if exist(cadgeofile) == 2,
        eval(['load ',cadgeofile])
    else,
        error(['ERROR: The cadgeo file named ',cadgeofile,' could not be opened'])            
    end
end
%--------------------------------------------------------------

[ncorners,slask] = size(corners);
[nedges,slask] = size(edgecorners);
[nplanes,slask] = size(planenvecs);

planelist = 1:nplanes;

viewpos = EDBmyvalueinput('From which point do you want to watch the model? (x y z, or az el, with spaces inbetween) ',-1);

%plot3(receivers(:,1),receivers(:,2),receivers(:,3),'o');

% figure(1)
hold off
for ii = 1:nedges,
	co1 = edgecorners(ii,1);
	co2 = edgecorners(ii,2);
	iv = [co1;co2];
	plot3(corners(iv,1),corners(iv,2),corners(iv,3))
    if ii ==1,
        view(viewpos)
        hold
    end
end

if plotnvecs,
	for ii = 1:nplanes,
        midpoint = mean(corners(planecorners(ii,1:(ncornersperplanevec(ii))),:));
        
        endpoint = midpoint + planenvecs(ii,:);
        bothpoints = [midpoint;endpoint];
        plot3(bothpoints(:,1),bothpoints(:,2),bothpoints(:,3));
        plot3(midpoint(1),midpoint(2),midpoint(3),'ro');
	end
end

if plotplnumbers,
	for ii = 1:nplanes,
        midpoint = mean(corners(planecorners(ii,1:ncornersperplanevec(ii)),:));
        endpoint = midpoint + planenvecs(ii,:);
        if plotplCADnumbers == 1,
            text(endpoint(1),endpoint(2),endpoint(3),int2str(planenumbers(ii)));
        else,
            text(endpoint(1),endpoint(2),endpoint(3),int2str(ii));
        end
    
	end
end

if plotednumbers,
    for ii = 1:nedges,
        midpoint = mean(corners(edgecorners(ii,1:2),:));
        text(midpoint(1),midpoint(2),midpoint(3),int2str(ii));
    end
end

if plotconumbers,
    for ii = 1:ncorners,
        if plotcoCADnumbers == 1,
            text(corners(ii,1),corners(ii,2),corners(ii,3),int2str(cornernumbers(ii)))    
        else,
            text(corners(ii,1),corners(ii,2),corners(ii,3),int2str(ii))    
        end
    end
    
end

if plotsources == 1,
    plot3(sources(:,1),sources(:,2),sources(:,3),'*');
end

if plotreceivers == 1,
    plot3(receivers(:,1),receivers(:,2),receivers(:,3),'ro');
end
