function [outputfile] = EDBrefinecad(CADfile,outputfile,maxedgelength);
% EDBrefinecad - Reads a file of type .CAD and refines edges automatically.
% EDBrefinecad reads a file of type .CAD (made by e.g. CATT-Acoustic) and
% checks all edge lengths. If any edge is longer than a specified
% maxedgelength, extra corners are inserted. A new CAD-file is created,
% either with an automatic file name, or one specified by the user.
% CAD-files v6,v7,v8 are read. The output file contains only corners and
% planes!
%
% Input parameters:
%  CADfile     	(optional) The input file, with or without the .CAD extension.
%					If this file is not specified, a file opening window will appear.
%  outputfile      (optional) The output file, including complete path. If not specified, it will be given the
%                   same name as the CAD file with an '_modified.cad' ending instead of the '.CAD'.
%  maxedgelength    The desired maxedgelength in meters.
%  SHOWTEXT (global)	If this global parameter has the value 3 or higher, informative text will
%                   be printed on the screen.
%
% Output parameters:
%	outputfile		The name of the outputfile, generated either automatically or specified as 
%					an input parameter.
%
% Uses the functions EDBfistr, EDBextrnums, EDBcross
% 
% ----------------------------------------------------------------------------------------------
%   This file is part of the Edge Diffraction Toolbox by Peter Svensson.                       
%                                                                                              
%   The Edge Diffraction Toolbox is free software: you can redistribute it and/or modify       
%   it under the terms of the GNU General Public License as published by the Free Software     
%   Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                              
%   The Edge Diffraction Toolbox is distributed in the hope that it will be useful,       
%   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                              
%   You should have received a copy of the GNU General Public License along with the           
%   Edge Diffraction Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
% ----------------------------------------------------------------------------------------------
% Peter Svensson (svensson@iet.ntnu.no) 050504
%
% [outputfile] = EDBrefinecad(CADfile,outputfile,maxedgelength);

global SHOWTEXT

if nargin < 3,
    error('The parameter maxedgelength must be specified')
end

geomacc = 1e-10;

%---------------------------------------------------------------
% If no CAD-file was specified, present a file opening window

if isempty(CADfile),
	[CADfile,filepath] = uigetfile('*.cad','Please select the CADfile');
	if ~isstr(CADfile),
		return
	end
	[temp1,filestem,fileext,temp] = fileparts(CADfile);
else,
%	[filepath,CADfile] = EDBstrppath(CADfile);
	[filepath,filestem,fileext,temp] = fileparts(CADfile);
    CADfile = [filestem,fileext];
end

CADfile = [[filepath,filesep],filestem,'.CAD'];
if exist(CADfile) ~= 2,
    CADfile = [[filepath,filesep],filestem,'.cad'];
    if exist(CADfile) ~= 2,
    	error(['ERROR: CAD-file: ',CADfile,' can not be found'])
    end    
end

% if SHOWTEXT >= 3,
% 	disp(['      Reading from ',CADfile])
% end

%---------------------------------------------------------------
% If no output file was specified, construct an automatic file name

if isempty(outputfile),
	outputfile = [[filepath,filesep],filestem,'_modified.cad'];
end

%---------------------------------------------------------------
% Read in the entire file into the string B
% Find the line starts and ends

fid = fopen([CADfile],'r');

if fid == 0,
	error(['ERROR: The CADfile ',CADfile,' could not be opened.'])
end
B = fread(fid,inf,'char').';
fclose(fid);

% Cut out sequential spaces to save some space.
iv = find(B(1:length(B)-1)==32 & B(2:length(B))==32);
B(iv) = [];

% If the text file was generated on a Mac, each line has ASCII 13 at the end.
% On a PC, each line ends with ASCII 13, and the next line starts with ASCII 10.
% In Unix, each line ends with ASCII 10.
%
% Change this to a single ASCII 13

if 1==2,
	ASCII13vec = find(B==13);
	ASCII10vec = find(B==10);

	if ~isempty(ASCII13vec),
		if ~isempty(ASCII10vec),
			B(ASCII10vec) = [];
			CRpositionvec = find(B==13);
		else,
			CRpositionvec = ASCII13vec;
		end
	else,
		CRpositionvec = ASCII10vec;
	end
else,
	ASCII10vec = find(B==10);
	B(ASCII10vec) = B(ASCII10vec)*0 + 13;	

	% Cut out sequential CR to save some space.
	iv = find(B(1:length(B)-1)==13 & B(2:length(B))==13);
	B(iv) = [];

	% Cut out initial spaces to save some space.
	iv = find(B(1:length(B)-1)==13 & B(2:length(B))==32);
	B(iv+1) = [];
	CRpositionvec = find(B==13);
end

% Some special characters might give negative ASCII values

iv = find(B<0);
if ~isempty(iv),
	B(iv) = B(iv)+256;   
end

nBchars = length(B);

nlines = length(CRpositionvec);
if CRpositionvec(nlines) ~= nBchars,
	CRpositionvec = [CRpositionvec nBchars];
	nlines = nlines + 1;
end
linestartvec = [1 CRpositionvec(1:nlines-1) + 1];
ncharsperline = CRpositionvec - linestartvec;

spacelines = find( ncharsperline < 5 );
CRpositionvec(spacelines) = [];
linestartvec(spacelines) = [];
ncharsperline(spacelines) = [];
nlines = length(CRpositionvec);

Blowcaps = lower(B);

%---------------------------------------------------------------
% Look for the text 'CATT-Acoustic v' in the first line

Str = B( 1:CRpositionvec(1) );
iv = findstr('catt-acoustic v',lower(Str));
if isempty(iv),
	CATTversionnumber = [];
else,
	CATTversionnumber = str2num(setstr(Str(iv(1)+15)));
end

%---------------------------------------------------------------
% Find the four sections of the file:
%    %CORNERS      %PLANES      %SOURCES   %RECEIVERS
% (or small letters)
% We assume that they come in this order

%disp('Searching for corners')

nlinescut = round(nlines/10);
ilineCORNERS = EDBfistr(Blowcaps(1:CRpositionvec(nlinescut)),1,linestartvec(1:nlinescut),CRpositionvec(1:nlinescut),'%corners');
if ilineCORNERS == -1,
   ilineCORNERS = EDBfistr(Blowcaps,1,linestartvec,CRpositionvec,'%corners');
	if ilineCORNERS == -1,
	   error('ERROR: The .CAD file must contain the word %CORNERS');
	end      
end

%disp('Searching for planes')
ilinePLANES = EDBfistr((Blowcaps),ilineCORNERS+1,linestartvec,CRpositionvec,'%planes');
if ilinePLANES == -1,
	error('ERROR: The .CAD file must contain the word %PLANES');
end

%disp('Searching for sources')
ilineSOURCES = EDBfistr((Blowcaps),ilinePLANES+1,linestartvec,CRpositionvec,'%sources');
if ilineSOURCES == -1,
	ilineSOURCES = 0;
end

%disp('Searching for receivers')
ilineRECEIVERS = EDBfistr((Blowcaps),max([ilineSOURCES+1 ilinePLANES+1]),linestartvec,CRpositionvec,'%receivers');
if ilineRECEIVERS == -1,
	ilineRECEIVERS = 0;
end

%---------------------------------------------------------------
% Look for the corners definitions between the lines containing
% %SOURCES and %PLANES
% The lines should look like:
%	1   -0.2000   0.34   2.3

%%disp('CORNERS')
iv = [ilineCORNERS+1:ilinePLANES-1];

ncorners = length(iv);
corners = zeros(ncorners,3);
cornernumbers = zeros(ncorners,1);

for ii = 1:ncorners,
	iline = iv(ii);
	Str = B( linestartvec(iline):CRpositionvec(iline)-1 );
	numvect = EDBextrnums(setstr(Str));
	cornernumbers(ii) = numvect(1);
	corners(ii,:) = numvect(2:4);
end

%---------------------------------------------------------------
% Look for the planes definitions between the lines containing
% %PLANES and %SOURCES, or the end
% The lines should look like:
%  1 / planename /RIGID
%  1  4  3  2
%  2 / /RIGID

%%disp('PLANES')
i2 = ilineSOURCES + (nlines+1)*(ilineSOURCES==0);
iv = [ilinePLANES+1:i2-1];

nplanes = floor(length(iv)/2);
planecorners = zeros(nplanes,4);
planenames = zeros(nplanes,30);
planeabstypes = zeros(nplanes,30);
planenumbers = zeros(nplanes,1);
ncornersperplanevec = zeros(nplanes,1);

for iplane = 1:nplanes,
	iline = iv( (iplane-1)*2 + 1);
	Str = B( linestartvec(iline):CRpositionvec(iline)-1 );
   
	ivec = find( Str == 47 );
	if length(ivec) < 2,
		error(['ERROR: Text line for plane ',int2str(iplane),'erroneous (consecutive numbering): Must have two /'])
	end
   
   numvect = EDBextrnums(setstr(Str(1:ivec(1)-1)));
	planenumbers(iplane) = numvect(1);
	textstr = strtrim(setstr(Str( ivec(1)+1 : ivec(2)-1 )));
	if ~isempty(textstr),
%		planenames(iplane,1:length(textstr)) = double(textstr);
		planenames(iplane,1:length(textstr)) = textstr;
	end

	textstr = strtrim(setstr(Str( ivec(2)+1 : length(Str) )));
	if ~isempty(textstr),
%		planeabstypes(iplane,1:length(textstr)) = double(textstr);
		planeabstypes(iplane,1:length(textstr)) = textstr;
	end

	iline = iv( (iplane-1)*2 + 2);
	Str = B( linestartvec(iline):CRpositionvec(iline)-1 );
	numvect = EDBextrnums(setstr(Str));
% In the previous versions of readcad,EDreadcad etc, the corner order fromthe CAD file
% was reversed. 000802: Reinserted...and removed again
	%numvect = numvect(length(numvect):-1:1);  % 000802: Reinserted
	ncornersperplanevec(iplane) = length(numvect);
	[slask,ncornersperplane] = size(planecorners);
	if length(numvect) <= ncornersperplane,
		numvect = [numvect zeros(1,ncornersperplane-length(numvect))];
		planecorners(iplane,:) = numvect;
	else,
		nmorezeros = length(numvect) - ncornersperplane;
		planecorners = [planecorners zeros(nplanes,nmorezeros)];
		planecorners(iplane,:) = numvect;
	end
end

if max(max(planecorners)) > max(cornernumbers),
    error('ERROR: One plane definition in the CAD-file used a higher corner number than was defined in the CORNERS section')
end

%---------------------------------------------------------------
% Look for the sources definitions between the lines containing
% %SOURCES and %RECEIVERS, or the end
% The lines should look like: (v6)
% 0 OMNI:SD0
%   0.0000000   0.0000000   1.0000000
%   0.0000000   0.0000000   0.0000000
%  85.0  88.0  91.0  94.0  97.0  100.0
%
% or: (v7)
% A0 OMNI:SD0
%   0.0000000   0.0000000   1.0000000
%   0.0000000   0.0000000   0.0000000  15.00000
%  85.0  88.0  91.0  94.0  97.0  100.0 : 103.0 106.0

%%disp('SOURCES')
CATTversionnumber = [];

% This initial creation of empty matrices is to return something even if there
% are no sources in the file.

Snames = []; Snumbers = [];
Sdirectivitynames = [];
Sdelays = [];
Scoords = [];
Sdirections = [];
Srotations = [];
Slevels = [];

if ilineSOURCES ~= 0,
	i2 = ilineRECEIVERS + (nlines+1)*(ilineRECEIVERS==0);
	iv = [ilineSOURCES+1:i2-1];

	nsources = floor(length(iv)/4);
	Snumbers = zeros(nsources,1);
	Snames = zeros(nsources,4);
	Sdirectivitynames = ( zeros(nsources,30) );
	Sdelays = zeros(nsources,1);
	Scoords = zeros(nsources,3);
	Sdirections = zeros(nsources,3);
	Srotations = zeros(nsources,1);
	Slevels = zeros(nsources,8);

	for isource = 1:nsources,
		iline = iv( (isource-1)*4+1);
		Str = strtrim(setstr( B(linestartvec(iline):CRpositionvec(iline)-1 )) );
		ivec = find( Str == 32);
		if isempty(ivec),
			sounameornumber = setstr(Str);
			textstr = '';
			delayval = 0;
		else,
			sounameornumber = setstr(Str( 1:ivec(1)-1));
			textstr = strtrim(setstr(Str(ivec(1)+1:length(Str)) ));
			ivec = find( textstr == 58);   % :
			if length(textstr) > ivec(1) + 3,
				delayval = EDBextrnums( setstr(textstr(ivec(1) + 4:length(textstr) )) );
				textstr = textstr(1:ivec(1)+3);
			else,
				delayval = 0;
			end

		end

		if isource == 1,
			sounumb = str2num(sounameornumber);
			if ~isempty(sounumb),
				CATTversionnumber = 6;
			else,
				CATTversionnumber = 7;
			end
		end

		if CATTversionnumber == 6,
			Snumbers(isource) = str2num(sounameornumber);
		else,
%			Snames(isource,1:length(sounameornumber)) = double(sounameornumber);
			Snames(isource,1:length(sounameornumber)) = sounameornumber;
		end

%		Sdirectivitynames(isource,1:length(textstr)) = double(textstr);
		Sdirectivitynames(isource,1:length(textstr)) = textstr;
		Sdelays(isource) = delayval(1);

		iline = iv( (isource-1)*4+2);
		Str = B( linestartvec(iline):CRpositionvec(iline)-1 );
		numvect = EDBextrnums(setstr(Str));
		Scoords(isource,:) = numvect;

		iline = iv( (isource-1)*4+3);
		Str = B( linestartvec(iline):CRpositionvec(iline)-1 );
		numvect = EDBextrnums(setstr(Str));
		Sdirections(isource,:) = numvect(1:3);
		if CATTversionnumber == 7,
			Srotations(isource) = numvect(4);
		end

		iline = iv( (isource-1)*4+4);
		Str = B( linestartvec(iline):CRpositionvec(iline)-1 );

		numvect = EDBextrnums(setstr(Str));
		Slevels(isource,1:6) = numvect(1:6);
		if CATTversionnumber == 7,
			Slevels(isource,7:8) = numvect(7:8);
		end
	end

	if ~isempty(CATTversionnumber),
		if CATTversionnumber == 6,
			Snames = [];
			Srotations = [];
		else,
			Snumbers = [];
		end
	else,
		Snumbers = [];
	end
end

%---------------------------------------------------------------
% Look for the receivers definitions after the line containing
% %RECEIVERS
% The lines should look like:
%	1   -0.2000   0.34   2.3

% This initial creation of empty matrices is to return something evne if there
% are no sources in the file.

Rnumbers = [];    Rcoords = [];

if ilineRECEIVERS ~= 0 ,
	iv = [ilineRECEIVERS+1:nlines];

	nreceivers = length(iv);
	Rnumbers = zeros(nreceivers,1);
	Rcoords = zeros(nreceivers,3);

	for ii = 1:nreceivers,
		iline = iv(ii);
		Str = B( linestartvec(iline):CRpositionvec(iline)-1 );
		numvect = EDBextrnums(setstr(Str));
		Rnumbers(ii) = numvect(1);
		Rcoords(ii,:) = numvect(2:4);
	end

end

%---------------------------------------------------------------
% Go through all planes and check the corner-to-corner distances

extracornersneeded = 0;
for ii = 1:nplanes,
    maxconumber = max(cornernumbers);
    newconumbers = [];
    newcorners = [];
    if SHOWTEXT >= 3,
        disp(['Plane number ',int2str(ii)])
    end
   cornnumbs = planecorners(ii,:);
   cornnumbs = [cornnumbs cornnumbs(1)];
   corntocorndist = sqrt(sum( ( (   corners(cornnumbs(2:end),:)-corners(cornnumbs(1:end-1),:) ).').^2 ));
   iv = find(corntocorndist > maxedgelength);
   if ~isempty(iv),
       % Right away, we can check which planes share an edge with this
       % plane.
       possplanes = [];
            for jj = [ii+1:nplanes],
                numbofsharedcorners = length(intersect(planecorners(ii,:),planecorners(jj,:)));
                if numbofsharedcorners > 2,
%                     disp(['Plane ',int2str(jj),'!'])    
                    possplanes = [possplanes jj];
                end
            end
       ncorners = floor(corntocorndist/maxedgelength-1e-6);
       extracornersneeded = 1;
%        disp('Extra corners needed between:')        
%        [cornnumbs(1:end-1).' cornnumbs(2:end).' ncorners.']
       totnewcorners = sum(ncorners);
        if SHOWTEXT >= 3,
            disp(['   ',int2str(totnewcorners),' new corners needed'])
        end
       newconumbercounter = length(newconumbers)+1;
       if length(newconumbers) == 0,
            maxnewnumber = 0;
        else,
            maxnewnumber = newconumbers(end);
       end
       newconumbers = [newconumbers;[maxnewnumber+1:maxnewnumber+totnewcorners].'];
       newcorners = [newcorners;zeros(totnewcorners,3)];
       newplanecornerdef = planecorners(ii,:);
       jjcounter = 0;
       for jj = iv,
%           disp([int2str(ncorners(jj)),' new corners between corners ',int2str(cornnumbs(jj)),' and ',int2str(cornnumbs(jj+1))])
           edgelength = corntocorndist(jj);
           if ncorners(jj) == 1,
               newcornersforoneedge = corners(cornnumbs(jj),:) + corntocorndist(jj)/2*(corners(cornnumbs(jj+1),:)-corners(cornnumbs(jj),:));
           else,
               startcorner = corners(cornnumbs(jj),:);
               edgevec = corners(cornnumbs(jj+1),:)-corners(cornnumbs(jj),:);
               countvec = [1:ncorners(jj)].';
               newcornersforoneedge = startcorner(ones(ncorners(jj),1),:) + countvec(:,ones(1,3))/(ncorners(jj)+1).*edgevec(ones(ncorners(jj),1),:);
           end
           newcorners(newconumbercounter:newconumbercounter+ncorners(jj)-1,:) = newcornersforoneedge;
           numlisttoinsert = newconumbers(newconumbercounter:newconumbercounter+ncorners(jj)-1).'+maxconumber;
           newplanecornerdef = [ newplanecornerdef(1:jj+jjcounter) numlisttoinsert newplanecornerdef(jj+jjcounter+1:end)    ];
           ncornersperplanevec(ii) = ncornersperplanevec(ii)+ncorners(jj); 
           newconumbercounter = newconumbercounter + ncorners(jj);
           
           if ~isempty(possplanes),
                for kk = 1:length(possplanes),
                    planetomod = possplanes(kk);
                    [slask,iv1,iv2] = intersect(planecorners(planetomod,:),[cornnumbs(jj+1:-1:jj)]);
                    if length(iv1) == 2,
                        if ncornersperplanevec(planetomod)+ncorners(jj) > size(planecorners,2),
                            planecorners = [planecorners zeros(nplanes,ncorners(jj))];    
                        end
                        iv1 = sort(iv1);
                        if diff(iv1) == 1,
                            planecorners(planetomod,1:ncornersperplanevec(planetomod)+length(numlisttoinsert)) = [planecorners(planetomod,1:iv1(1))  numlisttoinsert(end:-1:1)  planecorners(planetomod,iv1(2):ncornersperplanevec(planetomod))];
                        else,           
                            planecorners(planetomod,1:ncornersperplanevec(planetomod)+length(numlisttoinsert)) = [planecorners(planetomod,1:ncornersperplanevec(planetomod))  numlisttoinsert(end:-1:1)  ];                            
                        end
                        ncornersperplanevec(planetomod) = ncornersperplanevec(planetomod) + ncorners(jj);
                    end
                end
           end
           
           jjcounter = jjcounter+ncorners(jj);
       end
       
       if length(newplanecornerdef) > size(planecorners,2),
            planecorners = [planecorners,zeros(nplanes,size(planecorners,2)-length(newplanecornerdef))];            
       end
       planecorners(ii,1:length(newplanecornerdef)) = newplanecornerdef;
       corners = [corners;newcorners];       
   end
end

nplanes = size(planecorners,1);
ncorners = size(corners,1);

%------------------------------------------------------------------------------
% Create a CAD file

compstr = computer;
compstr = lower(compstr(1:3));
if compstr == 'mac',  %isppc == 1,
	Lineending = 13;
elseif compstr == 'sun' | compstr == 'sol',            % isunix == 1,
	Lineending = 10;
elseif compstr == 'pcw',
	Lineending = [13,10];
else,
		error('ERROR: Not implemented for this computer type yet')	
end

fid = fopen([outputfile],'w');
if fid == -1,
	disp('This file is not possible to open - check that it isn''t opened by any program!')
	return
end

%------------------------------------------------------------------------------

fwrite(fid,['% ',outputfile,' created on ',date,' by EDBrefinecad',Lineending],'char');
fwrite(fid,[' ',Lineending],'char');

%------------------------------------------------------------------------------
% Write the corners

fwrite(fid,['%CORNERS',Lineending],'char');
fwrite(fid,[' ',Lineending],'char');
for ii = 1: ncorners,
	fwrite(fid,	[int2str(ii),' ',num2str(corners(ii,1),12),' ',num2str(corners(ii,2),12),' ',num2str(corners(ii,3),12),Lineending],'char');
end

%------------------------------------------------------------------------------
% Write the planes

fwrite(fid,[' ',Lineending],'char');
fwrite(fid,['%PLANES',Lineending],'char');
fwrite(fid,[' ',Lineending],'char');

for ii = 1:nplanes,
    covec = int2str(planecorners(ii,1));
    for jj = 2:ncornersperplanevec(ii),
    	covec = [covec,' ',int2str(planecorners(ii,jj))];
    end
    fwrite(fid,	['1 / /RIGID',Lineending],'char');
    fwrite(fid,	[covec,Lineending],'char');
    fwrite(fid,[' ',Lineending],'char');
end

fwrite(fid,['%EOF',Lineending],'char');

fclose(fid);
